import discord
from discord.ext import commands

from cpl_core.dependency_injection import ServiceProviderABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_core.logging import LoggerABC
from cpl_query.extension import List
from modules_core.abc.module_abc import ModuleABC
from modules_core.abc.module_service_abc import ModuleServiceABC
from modules_core.events_enum import EventsEnum


class ModuleService(ModuleServiceABC):

    def __init__(self, logger: LoggerABC, services: ServiceProviderABC, env: ApplicationEnvironmentABC):
        self._logger = logger
        self._services = services
        self._env = env
        self._modules: List[ModuleABC] = List()
        self._modules.extend(ModuleABC.__subclasses__())

    async def on_ready(self):
        self._logger.debug(__name__, 'Start on_ready modules')
        modules = self._modules.where(lambda m: hasattr(m, EventsEnum.on_ready.value) and callable(m.on_ready))
        for module_type in modules:
            module = self._services.get_service(module_type)
            await module.on_ready()

    async def on_message(self, message: discord.Message):
        pass